from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class QueryStory(Query):
    """
    Class for obtaining available info for one or more stories (clusters) in the Event Registry
    NOTE: Story in our terminology is a cluster of articles (and not a single article). An event is
    then something that consists of one or more stories (typically in different languages).

    @param storyUriOrList: a single story uri or a list of story uris
    """
    def __init__(self, storyUriOrList = None):
        super(QueryStory, self).__init__()
        self._setVal("action", "getStory")
        if storyUriOrList != None:
            self.queryByUri(storyUriOrList)

    def _getPath(self):
        return "/json/story"

    def queryByUri(self, uriOrUriList):
        """search stories by their uri(s)"""
        self._setVal("storyUri", uriOrUriList)

    def queryById(self, idOrIdList):
        """specify stories by their id(s)"""
        self._setVal("storyId", idOrIdList)

    def addRequestedResult(self, requestStory):
        """
        Add a result type that you would like to be returned.
        In one QueryStory you can ask for multiple result types.
        Result types can be the classes that extend RequestStory base class (see classes below).
        """
        assert isinstance(requestStory, RequestStory), "QueryStory class can only accept result requests that are of type RequestStory"
        self.resultTypeList.append(requestStory)


class RequestStory:
    def __init__(self):
        self.resultType = None


class RequestStoryInfo(RequestStory):
    """
    return details about a story
    """
    def __init__(self, returnInfo = ReturnInfo()):
        self.resultType = "info"
        self.__dict__.update(returnInfo.getParams("info"))


class RequestStoryArticles(RequestStory):
    """
    return articles about the story
    """
    def __init__(self,
                 page = 1,              # page of the articles
                 count = 20,            # number of articles to return
                 lang = mainLangs,      # return articles in specified language(s)
                 sortBy = "cosSim", sortByAsc = False,              # order in which story articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to story centroid), socialScore (total shares in social media), facebookShares (shares on fb), twitterShares (shares on twitter)
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 200))):
        assert page >= 1, "page has to be >= 1"
        assert count <= 200
        self.resultType = "articles"
        self.articlesPage = page
        self.articlesCount = count
        self.articlesLang = lang
        self.articlesSortBy = sortBy
        self.articlesSortByAsc = sortByAsc
        self.__dict__.update(returnInfo.getParams("articles"))


class RequestStoryArticleUris(RequestStory):
    """
    return a list of article uris
    """
    def __init__(self,
                 lang = mainLangs,
                 sortBy = "cosSim", sortByAsc = False  # order in which story articles are sorted. Options: id (internal id), date (published date), cosSim (closeness to story centroid), socialScore (total shares in social media), facebookShares (shares on fb), twitterShares (shares on twitter)
                 ):
        self.articleUrisLang = lang
        self.articleUrisSortBy = sortBy
        self.articleUrisSortByAsc = sortByAsc
        self.resultType = "articleUris"


class RequestStoryArticleTrend(RequestStory):
    """
    return trending information for the articles about the story
    """
    def __init__(self,
                 lang = mainLangs,
                 minArticleCosSim = -1,
                 returnInfo = ReturnInfo(articleInfo = ArticleInfoFlags(bodyLen = 0))):
        self.resultType = "articleTrend"
        self.articleTrendLang = lang
        self.articleTrendMinArticleCosSim = minArticleCosSim
        self.__dict__.update(returnInfo.getParams("articleTrend"))


class RequestStorySimilarStories(RequestStory):
    """
    return a list of similar stories
    """
    def __init__(self,
                 count = 20,                    # number of similar stories to return
                 source = "concept",            # how to compute similarity. Options: concept cca
                 maxDayDiff = sys.maxsize,       # what is the maximum time difference between the similar stories and this one
                 returnInfo = ReturnInfo()):
        assert count <= 200
        self.resultType = "similarStories"
        self.similarEventsCount = count
        self.similarEventsSource = source
        if maxDayDiff != sys.maxsize:
            self.similarEventsMaxDayDiff = maxDayDiff
        self.similarEventsAddArticleTrendInfo = addArticleTrendInfo
        self.__dict__.update(returnInfo.getParams("similarEvents"))



"""
provides classes for getting new/updated events and articles
"""

from eventregistry.Base import *
from eventregistry.ReturnInfo import *

class GetRecentEvents(QueryParamsBase):
    def __init__(self,
            maxEventCount = 60,
            maxMinsBack = 10 * 60,
            mandatoryLang = None,
            mandatoryLocation = True,
            lastActivityId = 0,
            returnInfo = ReturnInfo()):
        """
        return info about recently modified events

        @param maxEventCount: determines the maximum number of events to return in a single call (max 250)
        @param maxMinsBack: sets how much in the history are we interested to look
        @param mandatoryLang: set a lang or array of langs if you wish to only get events covered at least by the specified language
        @param mandatoryLocation: if set to True then return only events that have a known geographic location
        @param lastActivityId: this is another way of settings how much in the history are we interested to look. Set when you have repeated calls of the method. Set it to lastActivityId obtained in the last response
        """
        QueryParamsBase.__init__(self)

        assert maxEventCount <= 1000
        self._setVal("action", "getRecentActivity")
        self._setVal("addEvents", True)
        self._setVal("addArticles", False)
        self._setVal("recentActivityEventsMaxEventCount", maxEventCount)
        self._setVal("recentActivityEventsMaxMinsBack", maxMinsBack)
        self._setVal("recentActivityEventsMandatoryLocation", mandatoryLocation)
        self._setVal("recentActivityEventsLastActivityId", lastActivityId)
        # return only events that have at least a story in the specified language
        if mandatoryLang != None:
            self._setVal("recentActivityEventsMandatoryLang", mandatoryLang)
        self._update(returnInfo.getParams("recentActivityEvents"))

    def _getPath(self):
        return "/json/overview"

    def getUpdates(self, er):
        """
        Get the latest new or updated events from Event Registry
        This method is to be called repeatedly, each it will return only events that were changed since the last call
        @param er: instance of EventRegistry class
        """
        # execute the query
        ret = er.execQuery(self)

        # extract the latest activity id and remember it for the next query
        if ret and ret.get("recentActivity") and ret["recentActivity"].get("events"):
            lastActivityId =  ret["recentActivity"]["events"].get("newestActivityId", 0)
            self._setVal("recentActivityEventsLastActivityId", lastActivityId)
            # return the updated information
            return ret["recentActivity"]["events"]
        # or empty
        return {}


class GetRecentArticles(QueryParamsBase):
    def __init__(self,
            maxArticleCount = 60,
            maxMinsBack = 10 * 60,
            mandatorySourceLocation = False,
            lastActivityId = 0,
            returnInfo = ReturnInfo()):
        """
        return info about recently added articles
        @param maxArticleCount: determines the maximum number of articles to return in a single call (max 250)
        @param maxMinsBack: sets how much in the history are we interested to look
        @param mandatorySourceLocation: if True then return only articles from sources for which we know geographic location
        @param lastActivityId: another way of settings how much in the history are we interested to look. Set when you have repeated calls of the method. Set it to lastActivityId obtained in the last response
        """
        QueryParamsBase.__init__(self)

        assert maxArticleCount <= 1000
        self._setVal("action", "getRecentActivity")
        self._setVal("addEvents", False)
        self._setVal("addArticles", True)
        self._setVal("recentActivityArticlesMaxArticleCount", maxArticleCount)
        self._setVal("recentActivityArticlesMaxMinsBack", maxMinsBack)
        self._setVal("recentActivityArticlesMandatorySourceLocation", mandatorySourceLocation)
        self._setVal("recentActivityArticlesLastActivityId", lastActivityId)
        self._update(returnInfo.getParams("recentActivityArticles"))

    def _getPath(self):
        return "/json/overview"

    def getUpdates(self, er):
        """
        Get the latest new or updated events articles Event Registry
        This method is to be called repeatedly, each it will return only articles that were added since the last call
        @param er: instance of EventRegistry class
        """
        # execute the query
        ret = er.execQuery(self)

        # extract the latest activity id and remember it for the next query
        if ret and ret.get("recentActivity") and ret["recentActivity"].get("articles"):
            lastActivityId = ret["recentActivity"]["articles"].get("lastActivityId", 0)
            self._setVal("recentActivityArticlesLastActivityId", lastActivityId)
            # return the latest articles
            return ret["recentActivity"]["articles"]["activity"]
        # or empty
        return []
